@@ -21,13 +21,18 @@ from gevent.event import AsyncResult
 #from pickle import loads as pickle_loads, dumps as pickle_dumps, HIGHEST_PROTOCOL
 from cPickle import loads as pickle_loads, dumps as pickle_dumps, HIGHEST_PROTOCOL
 
-try:
-    #warn:msgpack turn list to tuple
-    from msgpack import loads, dumps, load, dump
-    loads = functools.partial(loads, use_list=1)
-    #print 'use msgpack'
-except ImportError:
-    from cPickle import loads, dumps, load, dump
+from msgpack import loads, dumps, load
+def use_list(use):
+    global loads, load
+    loads = functools.partial(loads, use_list=use)
+    load = functools.partial(load, use_list=use)
+use_list(True)
+
+#try:
+#    #warn:msgpack turn list to tuple
+#    #print 'use msgpack'
+#except ImportError:
+#    from cPickle import loads, dumps, load, dump
 
 
 from logging import root as logger, currentframe
@@ -396,14 +401,14 @@ class RpcService(object):
         self.stoped = True
         self._recv_task.kill(block=0)
         self._recv_task = None
-        self.svr.svc_stop(self)
         if 1 and not self.sock_error:
             try:
                 #printf('remote_stop:%s', self.sock_addr)
                 self.call('', 'remote_stop', tuple(), None, no_result=True)
-                sleep(0.05)
+                sleep(0.01)
             except:
                 pass
+        self.svr.svc_stop(self)
         if getattr(self, '_heart_task', None):
             self._heart_task.kill(block=False)
             self._heart_task = None
@@ -440,16 +445,17 @@ class RpcService(object):
 
     def _recver(self):
         """ 接收处理数据 """
+        recv_func = self.sock.recv
         def _read(c):
-            d = self.sock.recv(c)
-            if not d:
-                if self.stoped:
-                    raise GreenletExit
-                self._recver_on_error = True
-                self._on_socket_error(None)
-                self._recver_on_error = False
-                return None
-            return d
+            d = recv_func(c)
+            if d:
+                return d
+            if self.stoped:
+                raise GreenletExit
+            self._recver_on_error = True
+            self._on_socket_error(None)
+            self._recver_on_error = False
+            return None
 
         try:
             sio = StringIO()
@@ -554,27 +560,28 @@ class RpcService(object):
             raise error
         return resp
 
+    def _reg_obj(self, obj):
+        if hasattr(obj, 'proxy_pack'):
+            return obj.proxy_pack(), False
+        if isinstance(obj, RpcProxy):
+            return obj._id, False
+        if hasattr(obj, '_rpc_proxy_'):
+            return obj._rpc_proxy_(), True
+        return self.svr.register(obj), False
+
     def call(self, obj_id, name, args, kw, no_result=False,
             timeout=CALL_TIMEORUT, pickle=False, proxy=False):
         dtype = RT_REQUEST
         if proxy:
-            def _reg_obj(obj):
-                if hasattr(obj, 'proxy_pack'):
-                    return obj.proxy_pack(), False
-                if isinstance(obj, RpcProxy):
-                    return obj._id, False
-                if hasattr(obj, '_rpc_proxy_'):
-                    return obj._rpc_proxy_(), True
-                return self.svr.register(obj), False
             objs = args[0] #first arg is proxy(str, RpcProxy or list)
             if isinstance(objs, (tuple, list)):
                 obj_ids = []
                 for o in objs:
-                    obj, is_pickle = _reg_obj(o)
+                    obj, is_pickle = self._reg_obj(o)
                     pickle = pickle or is_pickle
                     obj_ids.append(obj)
             else:
-                obj, is_pickle = _reg_obj(objs)
+                obj, is_pickle = self._reg_obj(objs)
                 pickle = pickle or is_pickle
                 obj_ids = obj
             args = (obj_ids, ) + args[1:]
@@ -603,7 +610,7 @@ class RpcService(object):
             if obj is None:
                 raise RpcExportNoFound, obj_id
             func = getattr(obj, name)
-            if func is None or not callable(func):
+            if not callable(func):
                 raise RpcFuncNoFound, name
 
             if dtype & DT_ZIP:
@@ -612,21 +619,15 @@ class RpcService(object):
                 args, kw = pickle_loads(argkw)
             else:
                 args, kw = loads(argkw)
+
             if dtype & DT_PROXY:
-                def _get_proxy(export):
-                    if isinstance(export, RpcProxy):
-                        return export
-                    if isinstance(export, (tuple, list)):
-                        export_cls = PROXYS[export[0]]
-                        return export_cls.proxy_unpack(export, svc=self)
-                    return self.get_proxy(export)
                 export_ids = args[0]
                 if isinstance(export_ids, (tuple, list)):
                     proxys = []
                     for e in export_ids:
-                        proxys.append(_get_proxy(e))
+                        proxys.append(self.get_proxy(e))
                 else:
-                    proxys = _get_proxy(export_ids)
+                    proxys = self.get_proxy(export_ids)
                 args = (proxys, ) + tuple(args[1:])
 
             let = getcurrent()
@@ -719,6 +720,12 @@ class RpcService(object):
 
     def get_proxy(self, export_id, proxy_cls=None):
         """ remote call: get export obj by id """
+        if isinstance(export_id, RpcProxy):
+            return export_id
+        if isinstance(export_id, (tuple, list)):
+            export_cls = PROXYS[export_id[0]]
+            return export_cls.proxy_unpack(export_id, svc=self)
+
         if proxy_cls in (None, RpcProxy):
             try:
                 return self._proxys[export_id]
@@ -913,6 +920,7 @@ class RpcProxy(object):
         else:
             self._addr = self._svc.sock_addr
         self.timeout = CALL_TIMEORUT
+        self._methods = {}
         self._closes = {}#WeakKeyDictionary()
 
     def __reduce__(self):
@@ -937,11 +945,17 @@ class RpcProxy(object):
             no_result=no_result, timeout=timeout, pickle=pickle, proxy=proxy)
 
     def __getattr__(self, attribute):
-        def _func(*args, **kw):
-            no_result, timeout, pickle, proxy = parse_kw(kw, self.timeout)
-            return self._svc.call(self._id, attribute, args, kw,
-                no_result=no_result, timeout=timeout, pickle=pickle, proxy=proxy)
-        return _func
+        try:
+            return self._methods[attribute]
+        except KeyError:
+            def _func(*args, **kw):
+                no_result, timeout, pickle, proxy = parse_kw(kw, self.timeout)
+                return self._svc.call(self._id, attribute, args, kw,
+                    no_result=no_result, timeout=timeout, pickle=pickle, proxy=proxy)
+            _func.func_name = attribute
+            _func.__module__ = str(self._id)
+            self._methods[attribute] = _func
+            return _func
 
     def __getitem__(self, key):
         return self.__getattr__('__getitem__')(key)
