@@ -47,6 +47,7 @@ class RpcShell(code.InteractiveConsole):
             import app
         except ImportError:
             app = None
+        self.app = app
         shells = {'app': app}
         for name in dir(self):
             if name.startswith(self._PRE_SHELL_):
@@ -137,6 +138,8 @@ class RpcShell(code.InteractiveConsole):
 
     def shell_shell(self, proxy):
         """ 桥接shell """
+        if isinstance(proxy, (str, unicode)):
+            proxy = self.app.shell_get(proxy)
         addr = proxy.get_addr()
         svc = proxy.get_service()
         pre_prompt = str(addr)
